/******************************************************************************
 * @file     vio_S32K3X4EVB-Q172.c
 * @brief    Virtual I/O implementation for board S32K3X4EVB-Q172
 * @version  V2.0.0
 * @date     07. November 2023
 ******************************************************************************/
/*
 * Copyright (c) 2022-2024 Arm Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*! \page vio_S32K3X4EVB-Q172 Physical I/O Mapping
The table below lists the physical I/O mapping of this CMSIS-Driver VIO implementation.
Virtual Resource  | Variable       | Physical Resource on S32K3X4EVB-Q172           |
:-----------------|:---------------|:-----------------------------------------------|
vioBUTTON0        | vioSignalIn.0  | PTB19 (SW6, USER_SW1)                          |
vioBUTTON1        | vioSignalIn.1  | PTB26 (SW5, USER_SW0 disabled)                 |
vioLED0           | vioSignalOut.0 | PTA29 (RGBLED0_RED)                            |
vioLED1           | vioSignalOut.1 | PTA30 (RGBLED0_GREEN)                          |
vioLED2           | vioSignalOut.2 | PTA31 (RGBLED0_BLUE)                           |
*/

/* History:
 *  Version 2.0.0
 *    Updated to API 1.0.0
 *  Version 1.0.0
 *    Initial release
 */

#include <string.h>
#include "cmsis_vio.h"

#include "RTE_Components.h"                 // Component selection
#include CMSIS_device_header

#if !defined CMSIS_VOUT || !defined CMSIS_VIN
// Add user includes here:

#endif

// VIO input, output definitions
#define VIO_VALUE_NUM           3U          // Number of values

// VIO input, output variables
__USED uint32_t vioSignalIn;                // Memory for incoming signal
__USED uint32_t vioSignalOut;               // Memory for outgoing signal
__USED int32_t  vioValue[VIO_VALUE_NUM];    // Memory for value used in vioGetValue/vioSetValue

#if !defined CMSIS_VOUT
// Add global user types, variables, functions here:
static       volatile uint32_t * gpio_o_mscr[] = {&IP_SIUL2->MSCR[29], &IP_SIUL2->MSCR[30], &IP_SIUL2->MSCR[31] };
static       volatile uint8_t  * gpio_o_gpdo[] = {&IP_SIUL2->GPDO29,   &IP_SIUL2->GPDO30,   &IP_SIUL2->GPDO21 };
#endif

#if !defined CMSIS_VIN
// Add global user types, variables, functions here:
static       volatile uint32_t * gpio_i_mscr[] = {&IP_SIUL2->MSCR[51], &IP_SIUL2->MSCR[58] };
static const volatile uint8_t  * gpio_i_gpdi[] = {&IP_SIUL2->GPDI51,   &IP_SIUL2->GPDI58 };
#endif

// Initialize test input, output.
void vioInit (void) {
#if !defined CMSIS_VOUT
// Add user variables here:
uint32_t o;
#endif
#if !defined CMSIS_VIN
// Add user variables here:
uint32_t i;
#endif

  vioSignalIn  = 0U;
  vioSignalOut = 0U;

  memset(vioValue, 0, sizeof(vioValue));

#if !defined CMSIS_VOUT
  // Initialize LEDs pins
  for (o = 0; o < 3U; o++) {
    *gpio_o_mscr[o] = SIUL2_MSCR_OBE_MASK;   /* enable output */
    *gpio_o_gpdo[o] = 0U;                    /* LED off */
  }
#endif

#if !defined CMSIS_VIN
  // Initialize buttons pins
  for (i = 0; i < 2U; i++) {
    *gpio_i_mscr[i] = SIUL2_MSCR_IBE_MASK;   /* enable input */
  }
#endif
}

// Set signal output.
void vioSetSignal (uint32_t mask, uint32_t signal) {
#if !defined CMSIS_VOUT
// Add user variables here:

#endif

  vioSignalOut &= ~mask;
  vioSignalOut |=  mask & signal;

#if !defined CMSIS_VOUT
// Add user code here:

  if (mask & vioLED0) {
    if (signal & vioLED0) {
      *gpio_o_gpdo[0] = 1U;
    } else {
      *gpio_o_gpdo[0] = 0U;
    }
  }
  if (mask & vioLED1) {
    if (signal & vioLED1) {
      *gpio_o_gpdo[1] = 1U;
    } else {
      *gpio_o_gpdo[1] = 0U;
    }
  }
  if (mask & vioLED2) {
    if (signal & vioLED2) {
      *gpio_o_gpdo[2] = 1U;
    } else {
      *gpio_o_gpdo[2] = 0U;
    }
  }
#endif
}

// Get signal input.
uint32_t vioGetSignal (uint32_t mask) {
  uint32_t signal;
#if !defined CMSIS_VIN
// Add user variables here:

#endif

#if !defined CMSIS_VIN
// Add user code here:
  if (mask & vioBUTTON0) {
    if (*gpio_i_gpdi[0]) {
      vioSignalIn |=  vioBUTTON0;
    } else {
      vioSignalIn &= ~vioBUTTON0;
    }
  }
  if (mask & vioBUTTON1) {
    if (*gpio_i_gpdi[1]) {
      vioSignalIn |=  vioBUTTON1;
    } else {
      vioSignalIn &= ~vioBUTTON1;
    }
  }
#endif

  signal = vioSignalIn & mask;

  return signal;
}

// Set value output.
void vioSetValue (uint32_t id, int32_t value) {
  uint32_t index = id;
#if !defined CMSIS_VOUT
// Add user variables here:

#endif

  if (index >= VIO_VALUE_NUM) {
    return;                             /* return in case of out-of-range index */
  }

  vioValue[index] = value;

#if !defined CMSIS_VOUT
// Add user code here:

#endif
}

// Get value input.
int32_t vioGetValue (uint32_t id) {
  uint32_t index = id;
  int32_t  value = 0;
#if !defined CMSIS_VIN
// Add user variables here:

#endif

  if (index >= VIO_VALUE_NUM) {
    return value;                       /* return default in case of out-of-range index */
  }

#if !defined CMSIS_VIN
// Add user code here:

//   vioValue[index] = ...;
#endif

  value = vioValue[index];

  return value;
}
